@@ -11,7 +11,7 @@ use MIME::Base64;
 use strict;
 use vars qw($VERSION $AUTOLOAD);
 
-$VERSION = '0.07';
+$VERSION = '0.08';
 
 my $me = 'Apache::AuthTkt';
 my $PREFIX = 'TKTAuth';
@@ -204,24 +204,8 @@ sub ticket
 
     # Calculate the hash for the ticket
     my $ts = $arg{ts} || time;
-    my @ip = split /\./, $arg{ip_addr};
-    my @ts = ( (($ts & 0xff000000) >> 24),
-               (($ts & 0xff0000) >> 16),
-               (($ts & 0xff00) >> 8),
-               (($ts & 0xff)) );
-    my $ipts = pack("C8", @ip, @ts);
-    my $raw = $ipts . $self->secret . $arg{uid} . "\0" . $arg{tokens} . "\0" . $arg{data};
-    my $digest0 = md5_hex($raw);
-    my $digest = md5_hex($digest0 . $self->secret);
-
-    if ($arg{debug}) {
-        print STDERR "ts: $ts\nip_addr: $arg{ip_addr}\nuid: $arg{uid}\ntokens: $arg{tokens}\ndata: $arg{data}\n";
-        print STDERR "secret: " . $self->secret . "\n";
-        print STDERR "raw: '$raw'\n";
-        my $len = length($raw);
-        print STDERR "digest0: $digest0 (input length $len)\n";
-        print STDERR "digest: $digest\n";
-    }
+    my $digest = $self->_get_digest($ts, $arg{ip_addr}, $arg{uid}, $arg{tokens}, 
+        $arg{data}, $arg{debug});
 
     # Construct the ticket itself
     my $ticket = sprintf "%s%08x%s!", $digest, $ts, $arg{uid};
@@ -231,6 +215,31 @@ sub ticket
     return $arg{base64} ? encode_base64($ticket, '') : $ticket;
 }
 
+sub _get_digest
+{
+    my ($self, $ts, $ip_addr, $uid, $tokens, $data, $debug) = @_;
+    my @ip = split /\./, $ip_addr;
+    my @ts = ( (($ts & 0xff000000) >> 24),
+               (($ts & 0xff0000) >> 16),
+               (($ts & 0xff00) >> 8),
+               (($ts & 0xff)) );
+    my $ipts = pack("C8", @ip, @ts);
+    my $raw = $ipts . $self->secret . $uid . "\0" . $tokens . "\0" . $data;
+    my $digest0 = md5_hex($raw);
+    my $digest = md5_hex($digest0 . $self->secret);
+
+    if ($debug) {
+        print STDERR "ts: $ts\nip_addr: $ip_addr\nuid: $uid\ntokens: $tokens\ndata: $data\n";
+        print STDERR "secret: " . $self->secret . "\n";
+        print STDERR "raw: '$raw'\n";
+        my $len = length($raw);
+        print STDERR "digest0: $digest0 (input length $len)\n";
+        print STDERR "digest: $digest\n";
+    }
+
+    return $digest;
+}
+
 # Return a cookie containing a mod_auth_tkt ticket 
 sub cookie
 {
@@ -253,8 +262,31 @@ sub cookie
            $arg{cookie_name}, $ticket, $domain_elt, $path_elt, $secure_elt;
 }
 
-# returns a hashref representing the original ticket components
-# returns undef if there were any errors
+# Returns a hashref representing the original ticket components
+# Returns undef if there were any errors
+sub validate_ticket
+{
+    my $self = shift;
+    my $ticket = shift || croak "No ticket passed to validate_ticket";
+    my %arg = ( %$self, @_ );
+
+    $arg{ip_addr} = $arg{ignore_ip} ? '0.0.0.0' : $ENV{REMOTE_ADDR}
+        unless exists $arg{ip_addr};
+    # 0 or undef ip_addr treated as 0.0.0.0
+    $arg{ip_addr} ||= '0.0.0.0';
+
+    # Parse ticket
+    my $info = $self->parse_ticket($ticket);
+
+    # Validate digest
+    my $expected_digest = $self->_get_digest(
+        $info->{ts}, $arg{ip_addr}, $info->{uid},
+        $info->{tokens}, $info->{data});
+
+    return $info if $expected_digest eq $info->{digest};
+    return undef;
+}
+
 sub parse_ticket
 {
     my $self    = shift;
@@ -274,6 +306,7 @@ sub parse_ticket
     
     # Deconstruct
     my ($digest,$ts,$uid,$extra) = ($raw =~ m/^(.{32})(.{8})(.+?)!(.*)$/);
+    $parts->{digest} = $digest;
     $parts->{ts}  = hex($ts);
     $parts->{uid} = $uid;
     $parts->{tokens} = '';
@@ -291,6 +324,8 @@ sub parse_ticket
     return $parts;
 }
 
+# Alias for compatibility with Jose/Ton's original patch
+*valid_ticket = \&validate_ticket;
 
 1;
 
@@ -335,15 +370,15 @@ mod_auth_tkt apache module.
 
 =head1 INTRODUCTION
 
-Apache::AuthTkt is a module for generating authentication tickets
-for use with the 'mod_auth_tkt' apache module. Tickets are typically
-generated by a CGI/mod_perl page when a user has been authenticated.
-The ticket contains a username/uid for the authenticated user, and
-often also the IP address they authenticated from, a set of
-authorisation tokens, and any other user data required. The ticket
-also includes an MD5 hash of all the included user data plus a shared
-secret, so that tickets can be validated by mod_auth_tkt without
-requiring access to the user repository.
+Apache::AuthTkt is a module for generating and validating 
+authentication tickets used with the 'mod_auth_tkt' apache module. 
+Tickets are typically generated by a login web page of some kind 
+when a user has been authenticated. The ticket contains a username/uid 
+for the authenticated user, and often also the IP address they 
+authenticated from, a set of authorisation tokens, and any other user 
+data required. The ticket also includes an MD5 hash of all the included 
+user data plus a shared secret, so that tickets can be validated by 
+mod_auth_tkt without requiring access to the user repository.
 
 See http://www.openfusion.com.au/labs/mod_auth_tkt for mod_auth_tkt
 itself.
@@ -476,14 +511,18 @@ returned only in HTTPS contexts. Default: $at->require_ssl, or 0.
 
 =back
 
-=head2 TICKET VALIDATION
+=head2 TICKET PARSING AND VALIDATION
 
-You may validate a ticket with the parse_ticket() method. parse_ticket()
-takes one parameter (a ticket value) and returns a hashref with the following
-key/value pairs:
+You may parse and validate existing tickets with the validate_ticket() 
+method. It takes as its first parameter the ticket to be validated, and
+then an optional list of named parameter overrides 
+(e.g. ip_addr => 'x.x.x.x'). If the ticket is valid, validate_ticket 
+returns a hashref with the following key/value pairs:
 
 =over 4
 
+=item digest
+
 =item ts
 
 =item uid
@@ -494,11 +533,18 @@ key/value pairs:
 
 =back
 
-parse_ticket() will return undef if any errors with the ticket value are 
-encountered.
+validate_ticket() will return undef if any errors with the ticket value 
+are encountered.
+
+The validate_ticket() method algorithm is analogous to the function with
+the same name in the mod_auth_tkt C module.
+
+There is also a parse_ticket() method available that parses the ticket
+without running it through the validation phase, and returns the same
+data as validate_ticket(). This is only safe to use where you are certain
+that the ticket has been validated elsewhere. In general it's considerably
+safer to just use validate_ticket.
 
-The parse_ticket() method algorithm is analogous to the function with the 
-same name in the mod_auth_tkt C module.
 
 =head1 AUTHOR
 
@@ -508,12 +554,15 @@ Contributors:
 
 Peter Karman <peter@peknet.com>
 
+Ton Voon <ton.voon@altinity.com>
+
+Jose Luis Martinez <jlmartinez@capside.com>
 
 =head1 COPYRIGHT
 
-Copyright 2001-2008 Open Fusion Pty Ltd.
+Copyright 2001-2008 Gavin Carr and contributors.
 
-This program is free software. You may copy or redistribute it under the 
+This program is free software. You may copy or redistribute it under the
 same terms as perl itself.
 
 =cut
